function[out]=lnS_MAP(pars0,pars1,pars2,pars3);
%%%Function to compute the maximum a posteriori (MAP) estimates of parameters 
%in the 'lnS' model. This code is called under a code 'lnS'. The MAP 
%estimates are obtained using a matlab built-in function 'fminsearch'.

%%%Inputs:
%'pars0' is a vector collecting an initial guess for the MAP estimates of 
%the posterior distributions of the parameters in the 'lnS' model.
%'pars1' is a matrix collecting data for stock(s) standardized for data 
%analysis (data(:,1)=log(s)/(max(log(s))-min(log(s))) and data for
%recruitment in log scale(data(:,2)=log(r)),where 'r' is
%recruitment. 
%'pars2' is a vector specifying a prior distribution for 'phi'
%'pars3' is a vector specifying a prior distribution for 'tau squared'

%%%Outputs:
%'out' is a log(likelihood times prior) calculated while searching for a 
%combination of parameters in the 'lnS' model that minimizes 
%log(likelihood times prior).This does not appear as an output.

%%%Description of 'lmS_MAP'.
%Changing notations of parameters.
loga=pars0(1);
logphi=pars0(2);
logtau=pars0(3);
logv=pars0(4);

%Changing notations of data.
x=pars1(:,1);
y=pars1(:,2);

%Defining vectors and maxtices that are used for constructing mean and
%covariance functions in the 'lnS' model.
lx=length(x);
ly=length(y);
iy=eye(ly);
xd=abs(x*ones(1,lx)-ones(lx,1)*x').^2;

%Constructing the covariance function.
co=exp(logtau)*exp(-exp(logphi)*xd);  

%Likelihood.
L=-1/2*log(det(exp(logv)*iy+co))-1/2*(y-loga)'*inv(co+exp(logv)*iy)*(y-loga);

%Priors (flat priors for log(alpha) and log(v).Gamma priors for phi and tau2,which are rescaled in log scale). 
alpphi=pars2(1);bepphi=pars2(2);Plogphi=log((exp(logphi)).^(-alpphi).*exp(-bepphi.*(exp(logphi))));
alptau=pars3(1);beptau=pars3(2);Plogtau=log((exp(logtau)).^(-alptau).*exp(-beptau.*(exp(logtau))));
Plogv=0;
out=-(L+Plogphi+Plogtau+Plogv);